-- name: [CS] Hatsune Miku
-- description: The Cheerful Vocaloid, Hastune Miku, makes her debut on the big stage, but this time in a bit of low poly form, can Miku make her way to super stardom and defeat Bowser?\n\n\Made by VioletArts

local TEXT_MOD_NAME = "Hatsune Miku"

-- Stops mod from loading if Character Select isn't on
if not _G.charSelectExists then
    djui_popup_create("\\#ffffdc\\\n"..TEXT_MOD_NAME.."\nRequires the Character Select Mod\nto use as a Library!\n\nPlease turn on the Character Select Mod\nand Restart the Room!", 6)
    return 0
end

local E_MIKU_MODEL = smlua_model_util_get_id("miku_violet_geo")

local TEX_MIKU_ICON = get_texture_info("miku_violet_icon")

local VOICETABLE_MIKU = {
    [CHAR_SOUND_ATTACKED] = 'MikuAttacked.ogg',
    [CHAR_SOUND_COUGHING3] = 'MikuCough.ogg',
    [CHAR_SOUND_DOH] = 'MikuDoh.ogg',
    [CHAR_SOUND_DROWNING] = 'MikuDrown.ogg',
    [CHAR_SOUND_DYING] = 'MikuDead.ogg',
    [CHAR_SOUND_EEUH] = 'MikuEeuh.ogg',
    [CHAR_SOUND_GROUND_POUND_WAH] = 'MikuWah.ogg',
    [CHAR_SOUND_HAHA] = 'MikuHaha.ogg',
    [CHAR_SOUND_HAHA_2] = 'MikuHaha.ogg',
    [CHAR_SOUND_HERE_WE_GO] = 'MikuSugoi.ogg',
    [CHAR_SOUND_HOOHOO] = 'MikuHooHoo.ogg',
    [CHAR_SOUND_HRMM] = 'MikuGrab.ogg',
    [CHAR_SOUND_IMA_TIRED] = 'MikuTired.ogg',
    [CHAR_SOUND_LETS_A_GO] = 'MikuLetsDoThis.ogg',
    [CHAR_SOUND_MAMA_MIA] = 'MikuOhNo.ogg',
    [CHAR_SOUND_ON_FIRE] = 'MikuBurning.ogg',
    [CHAR_SOUND_OOOF] = 'MikuOof.ogg',
    [CHAR_SOUND_OOOF2] = 'MikuOof.ogg',
    [CHAR_SOUND_PANTING] = 'MikuPanting.ogg',
    [CHAR_SOUND_PANTING_COLD] = 'MikuPanting.ogg',
    [CHAR_SOUND_PUNCH_HOO] = 'MikuHoo.ogg',
    [CHAR_SOUND_PUNCH_WAH] = 'MikuWah.ogg',
    [CHAR_SOUND_PUNCH_YAH] = 'MikuYah.ogg',
    [CHAR_SOUND_SNORING1] = 'MikuSnore1.ogg',
    [CHAR_SOUND_SNORING2] = 'MikuSnore2.ogg',
    [CHAR_SOUND_SO_LONGA_BOWSER] = 'MikuBowser.ogg',
    [CHAR_SOUND_TWIRL_BOUNCE] = 'MikuBoing.ogg',
    [CHAR_SOUND_UH] = 'MikuUh.ogg',
    [CHAR_SOUND_UH2] = 'MikuUh.ogg',
    [CHAR_SOUND_UH2_2] = 'MikuUh.ogg',
    [CHAR_SOUND_WAAAOOOW] = 'MikuYell.ogg',
    [CHAR_SOUND_WHOA] = 'MikuWoah.ogg',
    [CHAR_SOUND_YAHOO] = 'MikuYahoo.ogg',
    [CHAR_SOUND_YAHOO_WAHA_YIPPEE] = {'MikuYahoo.ogg', 'MikuWahoo.ogg', 'MikuYippee.ogg'},
    [CHAR_SOUND_YAH_WAH_HOO] = {'MikuYah.ogg', 'MikuWah.ogg', 'MikuHoo.ogg'},
    [CHAR_SOUND_YAWNING] = 'MikuYawn.ogg',
}

local MIKU_PALETTE = {
    [PANTS] = "353535",
    [SHIRT] = "FFFFFF",
    [GLOVES] = "353535",
    [SHOES] = "353535",
    [HAIR] = "43B9FF",
    [SKIN] = "FED7A4",
    [CAP] = "43B9FF",
    [EMBLEM] = "43B9FF"
}

local MIKU_CAPMODELS = {
    normal = smlua_model_util_get_id("miku_violet_cap_geo"),
    wing = smlua_model_util_get_id("miku_violet_cap_wing_geo"),
    metal = smlua_model_util_get_id("miku_violet_cap_metal_geo"),
    metalWing = smlua_model_util_get_id("miku_violet_cap_metal_wing_geo"),
}

local charSelectLoaded = false
local function on_character_select_load()
    CT_MIKU = _G.charSelect.character_add("Hatsune Miku", {"The Cheerful Vocaloid, Hastune Miku,", "makes her debut on the", "big stage, but this time", "in a bit of low poly", "form, can Miku make her", "way to super stardom and", "defeat Bowser?"}, "Violet", {r = 67, g = 185, b = 255}, E_MIKU_MODEL, CT_TOAD, TEX_MIKU_ICON)
    _G.charSelect.character_add_voice(E_MIKU_MODEL, VOICETABLE_MIKU)
    _G.charSelect.character_add_palette_preset(E_MIKU_MODEL, MIKU_PALETTE)
    _G.charSelect.character_add_caps(E_MIKU_MODEL, MIKU_CAPMODELS)

    charSelectLoaded = true
end

local function on_character_sound(m, sound)
    if not charSelectLoaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_MIKU then return _G.charSelect.voice.sound(m, sound) end
end
local function on_character_snore(m)
    if not charSelectLoaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_MIKU then return _G.charSelect.voice.snore(m) end
end

hook_event(HOOK_ON_MODS_LOADED, on_character_select_load)
hook_event(HOOK_CHARACTER_SOUND, on_character_sound)
hook_event(HOOK_MARIO_UPDATE, on_character_snore)